/* diversity-apdb.h - DiversityApdb
 *
 * Copyright 2008 OpenMoko, Inc.
 * Authored by Chia-I Wu <olv@opeapdboko.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _DIVERSITY_APDB_H_
#define _DIVERSITY_APDB_H_

#define G_DISABLE_CAST_CHECKS

#include <glib.h>
#include <glib-object.h>
#include <diversity-ap.h>

G_BEGIN_DECLS

#define DIVERSITY_TYPE_APDB		(diversity_apdb_get_type())
#define DIVERSITY_APDB(obj)		(G_TYPE_CHECK_INSTANCE_CAST((obj), DIVERSITY_TYPE_APDB, DiversityApdb))
#define DIVERSITY_APDB_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST((klass), DIVERSITY_TYPE_APDB, DiversityApdbClass))
#define DIVERSITY_IS_APDB(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), DIVERSITY_TYPE_APDB))
#define DIVERSITY_IS_APDB_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), DIVERSITY_TYPE_APDB))
#define DIVERSITY_APDB_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), DIVERSITY_TYPE_APDB, DiversityApdbClass))

typedef struct _DiversityApdb DiversityApdb;
typedef struct _DiversityApdbClass DiversityApdbClass;

typedef struct {
	double lon, lat;
	const gchar *hw_addr;
	const gchar *ssid;
} ApDesc;

struct _DiversityApdb {
	GObject parent_object;
};

struct _DiversityApdbClass {
	GObjectClass parent_class;

	void (*ready)(DiversityApdb *apdb, guint version, gpointer user_data);
	void (*update_done)(DiversityApdb *apdb, gboolean success, gpointer user_data);
	void (*query_done)(DiversityApdb *apdb, GSList *aps, gpointer user_data);
};

GType diversity_apdb_get_type(void) G_GNUC_CONST;

DiversityApdb *diversity_apdb_new(const gchar *url);

void diversity_apdb_set_server(DiversityApdb *apdb, const gchar *url);
const gchar *diversity_apdb_get_server(DiversityApdb *apdb);
gboolean diversity_apdb_is_ready(DiversityApdb *apdb);

gboolean diversity_apdb_request_update(DiversityApdb *apdb, ApDesc *aps, guint num_aps);
void diversity_apdb_cancel_update(DiversityApdb *apdb);

gboolean diversity_apdb_request_query(DiversityApdb *apdb, gdouble lon, gdouble lat, gdouble radius);
void diversity_apdb_cancel_query(DiversityApdb *apdb);

G_END_DECLS

#endif /* _DIVERSITY_APDB_H_ */
