/* diversity-conn-iface.h - DiversityConnIface
 *
 * Copyright 2008 OpenMoko, Inc.
 * Authored by Chia-I Wu <olv@openmoko.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _DIVERSITY_CONN_IFACE_H_
#define _DIVERSITY_CONN_IFACE_H_

#define G_DISABLE_CAST_CHECKS

#include <glib.h>
#include <glib-object.h>

G_BEGIN_DECLS

#define DIVERSITY_TYPE_CONN_IFACE		(diversity_conn_iface_get_type())
#define DIVERSITY_CONN_IFACE(obj)		(G_TYPE_CHECK_INSTANCE_CAST((obj), DIVERSITY_TYPE_CONN_IFACE, DiversityConnIface))
#define DIVERSITY_CONN_IFACE_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST((klass), DIVERSITY_TYPE_CONN_IFACE, DiversityConnIfaceClass))
#define DIVERSITY_IS_CONN_IFACE(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), DIVERSITY_TYPE_CONN_IFACE))
#define DIVERSITY_IS_CONN_IFACE_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), DIVERSITY_TYPE_CONN_IFACE))
#define DIVERSITY_CONN_IFACE_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), DIVERSITY_TYPE_CONN_IFACE, DiversityConnIfaceClass))

typedef struct _DiversityConnIface DiversityConnIface;
typedef struct _DiversityConnIfaceClass DiversityConnIfaceClass;

typedef struct _DiversityConnNetwork DiversityConnNetwork;
struct _DiversityConnNetwork {
	GTime last_seen;
	gchar *essid;
	gchar *bssid;
	gchar *security;
	gchar *psk;
	guint signal;
};

typedef struct _DiversityConnIPv4 DiversityConnIPv4;
struct _DiversityConnIPv4 {
	guint address;
	guint gateway;
	guint netmask;
	gboolean dhcp;
};

typedef enum _DiversityConnIfaceType DiversityConnIfaceType;
enum _DiversityConnIfaceType {
	DIVERSITY_CONN_IFACE_TYPE_UNKNOWN,
	DIVERSITY_CONN_IFACE_TYPE_80203,
	DIVERSITY_CONN_IFACE_TYPE_80211,
	DIVERSITY_CONN_IFACE_TYPE_WIMAX,
	DIVERSITY_CONN_IFACE_TYPE_MODEM,
	DIVERSITY_CONN_IFACE_TYPE_BLUETOOTH,
};

typedef enum _DiversityConnIfaceState DiversityConnIfaceState;
enum _DiversityConnIfaceState {
	DIVERSITY_CONN_IFACE_STATE_UNKNOWN,
	DIVERSITY_CONN_IFACE_STATE_OFF,
	DIVERSITY_CONN_IFACE_STATE_ENABLED,
	DIVERSITY_CONN_IFACE_STATE_SCANNING,
	DIVERSITY_CONN_IFACE_STATE_CONNECT,
	DIVERSITY_CONN_IFACE_STATE_CONNECTED,
	DIVERSITY_CONN_IFACE_STATE_CARRIER,
	DIVERSITY_CONN_IFACE_STATE_CONFIGURE,
	DIVERSITY_CONN_IFACE_STATE_READY,
	DIVERSITY_CONN_IFACE_STATE_SHUTDOWN,
};

typedef enum _DiversityConnIfacePolicy DiversityConnIfacePolicy;
enum _DiversityConnIfacePolicy {
	DIVERSITY_CONN_IFACE_POLICY_UNKNOWN,
	DIVERSITY_CONN_IFACE_POLICY_OFF,
	DIVERSITY_CONN_IFACE_POLICY_IGNORE,
	DIVERSITY_CONN_IFACE_POLICY_AUTO,
	DIVERSITY_CONN_IFACE_POLICY_ASK,
};

struct _DiversityConnIface {
	GObject parent_object;
};

struct _DiversityConnIfaceClass {
	GObjectClass parent_class;

	void (*network_found)(DiversityConnIface *cface, DiversityConnNetwork *net, gpointer user_data);
	void (*network_changed)(DiversityConnIface *cface, DiversityConnNetwork *net, gpointer user_data);

	/* network-found is not easily usable */ 
	void (*ap_added)(DiversityConnIface *cface, DiversityConnNetwork *net, gpointer user_data);
	void (*ap_removed)(DiversityConnIface *cface, DiversityConnNetwork *net, gpointer user_data);
	void (*ap_changed)(DiversityConnIface *cface, DiversityConnNetwork *net, gpointer user_data);
};

GType diversity_conn_iface_get_type(void) G_GNUC_CONST;

DiversityConnIface *diversity_conn_iface_new(const gchar *objpath, GError **error);

DiversityConnIfaceType diversity_conn_iface_get_iface_type(DiversityConnIface *cface);
const gchar *diversity_conn_iface_get_object_path(DiversityConnIface *cface);

const gchar *diversity_conn_iface_get_driver(DiversityConnIface *cface);
const gchar *diversity_conn_iface_get_vendor(DiversityConnIface *cface);
const gchar *diversity_conn_iface_get_product(DiversityConnIface *cface);

guint diversity_conn_iface_get_signal(DiversityConnIface *cface);
DiversityConnIfaceState diversity_conn_iface_get_state(DiversityConnIface *cface);

void diversity_conn_iface_set_policy(DiversityConnIface *cface, DiversityConnIfacePolicy policy);
DiversityConnIfacePolicy diversity_conn_iface_get_policy(DiversityConnIface *cface);

const DiversityConnIPv4 *diversity_conn_iface_get_ipv4(DiversityConnIface *cface);

void diversity_conn_iface_set_network(DiversityConnIface *cface, DiversityConnNetwork *net);
const DiversityConnNetwork *diversity_conn_iface_get_network(DiversityConnIface *cface);

void diversity_conn_iface_scan(DiversityConnIface *cface);

G_END_DECLS

#endif /* _DIVERSITY_CONN_IFACE_H_ */
