/* control-net.c
 *
 * Copyright 2008 OpenMoko, Inc.
 * Authored by Chia-I Wu <olv@openmoko.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <dbus/dbus-glib.h>
#include <dbus/dbus-glib-bindings.h>
#include <dbus/dbus-glib-lowlevel.h>
#include "diversity-control.h"
#include "control-marshal.h"

static gboolean control_net_connect_ap(DiversityNet *net, gchar *IN_path, GError **error);
static gboolean control_net_get_ap(DiversityNet *net, gchar **OUT_path, GError **error);
static gboolean control_net_connect_wired(DiversityNet *net, GError **error);
#include "control-net-glue.h"

static gboolean control_net_connect_ap(DiversityNet *net, gchar *IN_path, GError **error)
{
	DiversityControl *ctrl =
		diversity_control_from_equipment(DIVERSITY_EQUIPMENT(net));
	DiversityObject *obj;
	gboolean ret;

	obj = diversity_control_object_lookup(ctrl, IN_path);
	if (!obj || obj->type != DIVERSITY_OBJECT_TYPE_AP) {
		g_set_error(error, DIVERSITY_CONTROL_ERROR, 0,
				"non-existing AP %s\n", IN_path);

		return FALSE;
	}

	ret = diversity_net_connect_ap(net, DIVERSITY_AP(obj), error);
	if (!ret)
		(*error)->domain = DIVERSITY_CONTROL_ERROR;

	return ret;
}

static gboolean control_net_get_ap(DiversityNet *net, gchar **OUT_path, GError **error)
{
	DiversityControl *ctrl =
		diversity_control_from_equipment(DIVERSITY_EQUIPMENT(net));
	DiversityControlExport *export;
	DiversityObject *ap;

	ap = (DiversityObject *) diversity_net_get_active_ap(net);
	if (!ap) {
		g_set_error(error, DIVERSITY_CONTROL_ERROR, 0,
				"no active AP");

		return FALSE;
	}

	export = diversity_control_object_export(ctrl, ap, NULL);

	g_object_unref(ap);

	if (!export) {
		g_set_error(error, DIVERSITY_CONTROL_ERROR, 0,
				"failed to export AP");

		return FALSE;
	}

	*OUT_path = g_strdup(export->path);

	return TRUE;
}

static gboolean control_net_connect_wired(DiversityNet *net, GError **error)
{
	gboolean ret;

	ret = diversity_net_connect_wired(net, error);
	if (!ret)
		(*error)->domain = DIVERSITY_CONTROL_ERROR;

	return ret;
}
