/* control-tag.c -
 *
 * Copyright 2008 OpenMoko, Inc.
 * Authored by Chia-I Wu <olv@openmoko.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <diversity-tag.h>
#include "diversity-control.h"

static gboolean control_tag_get_created(DiversityTag *tag, gint *OUT_created, GError **error);
static gboolean control_tag_set(DiversityTag *tag, const gchar *IN_key, const gchar *IN_val, GError **error);
static gboolean control_tag_get(DiversityTag *tag, const gchar *IN_key, gchar **OUT_val, GError **error);
static gboolean control_tag_get_dict(DiversityTag *tag, GHashTable **OUT_dict, GError **error);
#include "control-tag-glue.h"

static gboolean control_tag_get_created(DiversityTag *tag, gint *OUT_created, GError **error)
{
	*OUT_created = diversity_tag_get_created(tag);

	return TRUE;
}

static gboolean control_tag_set(DiversityTag *tag, const gchar *IN_key, const gchar *IN_val, GError **error)
{
	if (!IN_key) {
		g_set_error(error, DIVERSITY_CONTROL_ERROR, 0,
				"no key supplied");

		return FALSE;
	}

	diversity_tag_set(tag, IN_key, IN_val);
	diversity_object_touch(DIVERSITY_OBJECT(tag));

	return TRUE;
}

static gboolean control_tag_get(DiversityTag *tag, const gchar *IN_key, gchar **OUT_val, GError **error)
{
	const gchar *val;

	if (!IN_key) {
		g_set_error(error, DIVERSITY_CONTROL_ERROR, 0,
				"no key supplied");

		return FALSE;
	}

	val = diversity_tag_get(tag, IN_key);
	*OUT_val = g_strdup(val);

	return TRUE;
}

static void dict_copy(gpointer key, gpointer val, gpointer data)
{
	GHashTable *dict = data;

	g_hash_table_insert(dict, g_strdup(key), g_strdup(val));
}

static gboolean control_tag_get_dict(DiversityTag *tag, GHashTable **OUT_dict, GError **error)
{
	GHashTable *dict;

	dict = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
	diversity_tag_foreach(tag, dict_copy, dict);

	*OUT_dict = dict;

	return TRUE;
}
