/* diversity-iwlib.h - DiversityIwlib
 *
 * Copyright 2008 OpenMoko, Inc.
 * Authored by Chia-I Wu <olv@opeiwliboko.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _DIVERSITY_IWLIB_H_
#define _DIVERSITY_IWLIB_H_

#define G_DISABLE_CAST_CHECKS

#include <glib.h>
#include <glib-object.h>
#include <diversity-ap.h>

G_BEGIN_DECLS

#define DIVERSITY_TYPE_IWLIB		(diversity_iwlib_get_type())
#define DIVERSITY_IWLIB(obj)		(G_TYPE_CHECK_INSTANCE_CAST((obj), DIVERSITY_TYPE_IWLIB, DiversityIwlib))
#define DIVERSITY_IWLIB_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST((klass), DIVERSITY_TYPE_IWLIB, DiversityIwlibClass))
#define DIVERSITY_IS_IWLIB(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), DIVERSITY_TYPE_IWLIB))
#define DIVERSITY_IS_IWLIB_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), DIVERSITY_TYPE_IWLIB))
#define DIVERSITY_IWLIB_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), DIVERSITY_TYPE_IWLIB, DiversityIwlibClass))

typedef struct _DiversityIwlib DiversityIwlib;
typedef struct _DiversityIwlibClass DiversityIwlibClass;

enum {
	DIVERSITY_IWLIB_STATE_DISCONNECTED = 0,
	DIVERSITY_IWLIB_STATE_ASSOCIATING,
	DIVERSITY_IWLIB_STATE_ASSOCIATED,
};

struct _DiversityIwlib {
	GObject parent_object;
};

struct _DiversityIwlibClass {
	GObjectClass parent_class;

	void (*state_changed)(DiversityIwlib *iwlib, guint state, gpointer user_data);
	void (*scanned_ap)(DiversityIwlib *iwlib, GSList *aps, gpointer user_data);
};

GType diversity_iwlib_get_type(void) G_GNUC_CONST;

DiversityIwlib *diversity_iwlib_new(const gchar *iface);

guint diversity_iwlib_get_state(DiversityIwlib *iwlib);

gboolean diversity_iwlib_request_associate(DiversityIwlib *iwlib, DiversityAp *ap);
gboolean diversity_iwlib_cancel_associate(DiversityIwlib *iwlib);
DiversityAp *diversity_iwlib_get_associate(DiversityIwlib *iwlib);

gboolean diversity_iwlib_request_scan(DiversityIwlib *iwlib);
void diversity_iwlib_cancel_scan(DiversityIwlib *iwlib);
gboolean diversity_iwlib_is_scanning(DiversityIwlib *iwlib);

const gchar *diversity_iwlib_get_interface(DiversityIwlib *iwlib);

G_END_DECLS

#endif /* _DIVERSITY_IWLIB_H_ */
