/* diversity-net.h - DiversityNet
 *
 * Copyright 2008 OpenMoko, Inc.
 * Authored by Chia-I Wu <olv@openmoko.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _DIVERSITY_NET_H_
#define _DIVERSITY_NET_H_

#define G_DISABLE_CAST_CHECKS

#include <glib-object.h>
#include <glib.h>
#include <diversity-ap.h>

G_BEGIN_DECLS

#define DIVERSITY_TYPE_NET		(diversity_net_get_type())
#define DIVERSITY_NET(obj)		(G_TYPE_CHECK_INSTANCE_CAST((obj), DIVERSITY_TYPE_NET, DiversityNet))
#define DIVERSITY_IS_NET(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), DIVERSITY_TYPE_NET))
#define DIVERSITY_NET_GET_IFACE(obj)	(G_TYPE_INSTANCE_GET_INTERFACE((obj), DIVERSITY_TYPE_NET, DiversityNetIface))

typedef struct _DiversityNet DiversityNet; /* dummy */
typedef struct _DiversityNetIface DiversityNetIface;

typedef enum _DiversityNetState DiversityNetState;

enum _DiversityNetState {
	DIVERSITY_NET_STATE_UNKNOWN,
	DIVERSITY_NET_STATE_DOWN,
	DIVERSITY_NET_STATE_DISCONNECTED,
	DIVERSITY_NET_STATE_PREPARE,
	DIVERSITY_NET_STATE_CONFIG,
	DIVERSITY_NET_STATE_NEED_AUTH,
	DIVERSITY_NET_STATE_IP_CONFIG,
	DIVERSITY_NET_STATE_ACTIVATED,
	DIVERSITY_NET_STATE_FAILED,
	DIVERSITY_NET_STATE_CANCELLED,
	DIVERSITY_NET_STATE_VERIFIED,
};

struct _DiversityNetIface {
	GTypeInterface base_iface;

	/* methods */
	gboolean (*connect_ap)(DiversityNet *net, DiversityAp *ap, GError **error);
	gboolean (*connect_wired)(DiversityNet *net, GError **error);

	/* signals */
	void (*state_changed)(DiversityNet *net, DiversityNetState state);
};

GType diversity_net_get_type(void) G_GNUC_CONST;

gchar *diversity_net_get_interface(DiversityNet *net);
DiversityNetState diversity_net_get_state(DiversityNet *net);
guint diversity_net_get_ip4_address(DiversityNet *net);
DiversityAp *diversity_net_get_active_ap(DiversityNet *net);

gboolean diversity_net_connect_ap(DiversityNet *net, DiversityAp *ap, GError **error);
gboolean diversity_net_connect_wired(DiversityNet *net, GError **error);

G_END_DECLS

#endif /* _DIVERSITY_NET_H_ */
