/* diversity-net.c - DiversityNet
 *
 * Copyright 2008 OpenMoko, Inc.
 * Authored by Chia-I Wu <olv@openmoko.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <glib.h>
#include <diversity-equipment.h>
#include <diversity-net.h>
#include "diversity-marshal.h"

enum {
	STATE_CHANGED,
	LAST_SIGNAL
};

static guint net_signals[LAST_SIGNAL] = { 0 };

static void diversity_net_class_init(gpointer g_iface);

GType diversity_net_get_type(void)
{
	static GType net_type = 0;

	if (!net_type) {
		net_type = g_type_register_static_simple(G_TYPE_INTERFACE,
							 "DiversityNet",
							 sizeof(DiversityNetIface),
							 (GClassInitFunc) diversity_net_class_init,
							 0, NULL, 0);

		g_type_interface_add_prerequisite(net_type, G_TYPE_OBJECT);
	}

	return net_type;
}

static void diversity_net_class_init(gpointer g_iface)
{
	GType iface_type = G_TYPE_FROM_INTERFACE(g_iface);

	net_signals[STATE_CHANGED] =
		g_signal_new("state-changed",
			     iface_type,
			     G_SIGNAL_RUN_LAST,
			     G_STRUCT_OFFSET(DiversityNetIface, state_changed),
			     NULL, NULL,
			     g_cclosure_marshal_VOID__UINT,
			     G_TYPE_NONE, 1,
			     G_TYPE_UINT);

	g_object_interface_install_property(g_iface,
			g_param_spec_string("interface",
					    "Interface",
					    "Interface of the device",
					    "",
					    G_PARAM_READABLE));

	g_object_interface_install_property(g_iface,
			g_param_spec_uint("state",
					  "State",
					  "State of the device",
					  DIVERSITY_NET_STATE_UNKNOWN,
					  DIVERSITY_NET_STATE_VERIFIED,
					  DIVERSITY_NET_STATE_UNKNOWN,
					  G_PARAM_READABLE));

	g_object_interface_install_property(g_iface,
			g_param_spec_uint("ip4-address",
					  "IPv4 Address",
					  "IPv4 address of the device",
					  0,
					  0xffffffff,
					  0,
					  G_PARAM_READABLE));

	g_object_interface_install_property(g_iface,
			g_param_spec_object("active-ap",
					    "Active AP",
					    "Active AP of the device",
					    DIVERSITY_TYPE_AP,
					    G_PARAM_READABLE));
}

gboolean diversity_net_connect_ap(DiversityNet *net, DiversityAp *ap, GError **error)
{
	return DIVERSITY_NET_GET_IFACE(net)->connect_ap(net, ap, error);
}

gboolean diversity_net_connect_wired(DiversityNet *net, GError **error)
{
	return DIVERSITY_NET_GET_IFACE(net)->connect_wired(net, error);
}

gchar *diversity_net_get_interface(DiversityNet *net)
{
	gchar *iface;

	g_object_get(net, "interface", &iface, NULL);

	return iface;
}

DiversityNetState diversity_net_get_state(DiversityNet *net)
{
	DiversityNetState state;

	g_object_get(net, "state", &state, NULL);

	return state;
}

guint diversity_net_get_ip4_address(DiversityNet *net)
{
	guint ip4;

	g_object_get(net, "ip4-address", &ip4, NULL);

	return ip4;
}

DiversityAp *diversity_net_get_active_ap(DiversityNet *net)
{
	DiversityAp *ap;

	g_object_get(net, "active-ap", &ap, NULL);

	return ap;
}
